using System.Drawing;
using System.Drawing.Drawing2D;
using System;
using System.Collections;
using TopCoder.Rules.DependencyMap.View;

namespace TopCoder.Rules.DependencyMap.View.AccuracyTests
{
	/// <summary>
	/// Helper class to manually build a diagram.
	/// 
    /// @author cucu
    /// Copyright (c)2005, TopCoder, Inc. All rights reserved
    /// </summary>
    public class GraphicsHelper
    {
        private ShapeFactory sf = null;
        private IGraphics graphics = null; 
        private SizeF shapeSize;
        private int colPadding = 0;
        private int rowPadding = 0;
        private int margin = 0;
        private float arrowHeight = 0;
		private int maxHeight = 0;
		private int maxWidth = 0;
		
		public int GetWidth() 
		{
			return maxWidth;
		}
    
		public int GetHeight() 
		{
			return maxHeight;
		}

        public SizeF ShapeSize
        {
            set 
            {
                shapeSize = value;
            }
        }
		
        /// <summary>
		/// get the graphics used for drawing
		/// </summary>
        public IGraphics Graphics 
        {
            get 
            {
                return graphics;
            }
        }
        /// <summary>
        /// Build a new Helper.
        /// </summary>
        /// <param name="shapes">shapes used for drawing</param>
        /// <param name="graphics">where to write output</param>
        public GraphicsHelper(ShapeFactory shapes, IGraphics graphics, int colPadding, int rowPadding, int margin)
        {
            sf = shapes;
            this.graphics = graphics;
            this.colPadding = colPadding;
            this.rowPadding = rowPadding;
            this.margin = margin;
            arrowHeight = shapes[ShapeFactory.RelationshipArrow].GetBounds().Height;
        }
        /// <summary>
        /// Convert from grid coordinates to pixel coordinates
        /// </summary>
        /// <param name="x">x in grid coordinates, 1-based</param>
        /// <param name="y">y in grid coordinates, 1-based</param>
        /// <returns>the point in pixel coordinates</returns>
        private PointF Convert(int x, int y)
        {
            return new PointF((x - 1) * (shapeSize.Width + colPadding), (y - 1) * (shapeSize.Height + arrowHeight + rowPadding));
        }

        /// <summary>
        /// Draw a seen/unseen shape with the specified text in the grid coordinates
        /// </summary>
        /// <param name="seen">true for seen shape</param>
        /// <param name="x">x in grid coordinates, 1-based</param>
        /// <param name="y">y in grid coordinates, 1-based</param>
        /// <param name="text">text to write centered</param>
        public void DrawShape(bool seen, int x, int y, string text) 
        {
            PointF p = Convert(x,y);
            DrawShape(seen, p.X, p.Y, shapeSize.Width, shapeSize.Height, text);
        }
        /// <summary>
        /// Draw an arrow that points from the x0,y0 grid coordinates to x1,y1 grid coordinates.
        /// 
        /// </summary>
        /// <param name="x0">start x in grid coordinates, 1-based</param>
        /// <param name="y0">start y in grid coordinates, 1-based</param>
        /// <param name="x0">end x in grid coordinates, 1-based</param>
        /// <param name="y0">end y in grid coordinates, 1-based</param>
        public void DrawArrow(int x0, int y0, int x1, int y1) 
        {
            PointF p0 =  Convert(x0, y0);
            PointF p1 =  Convert(x1, y1);
            p0.X += shapeSize.Width / 2;
            p0.Y += shapeSize.Height / 2;
            p1.X += shapeSize.Width / 2;
            p1.Y += shapeSize.Height / 2;
            DrawArrow(p0, p1);
        }
        
        /// <summary>
        /// Draw a shape in the specified pixel coordinates.
        /// </summary>
        /// <param name="seen">true for seen shape</param>
        /// <param name="x">left coordinate in pixels</param>
        /// <param name="y">upper coordinate in pixels</param>
        /// <param name="width">width of the shape</param>
        /// <param name="height">height of the shape</param>
        /// <param name="text">text to write centered in the shape</param>
        private void DrawShape(bool seen, float x, float y, float width, float height, string text)
        {
			if ((x + width) > maxWidth) 
			{
				maxWidth = (int) (x + width);
			}
			if ((y + height) > maxHeight) 
			{
				maxHeight = (int) (y + height);
			}

			GraphicsPath gp = sf [seen? ShapeFactory.SeenElement : ShapeFactory.UnseenElement];
            RectangleF bounds = gp.GetBounds();
            gp.Transform(new Matrix(width / bounds.Width, 0, 0, height / bounds.Height, x - bounds.X, y - bounds.Y));
            
            
            Pen pen= new Pen(Brushes.Black, 1);
            Font font = new Font(FontFamily.GenericSansSerif, 20);
            
            SizeF textSize = graphics.MeasureString(text, font);
            PointF textPos = new PointF(x + (width - textSize.Width) / 2, y + (height + textSize.Height) / 2);

            graphics.DrawPath(pen, gp);
            graphics.DrawString(text, font, Brushes.Black, textPos);
        }

        /// <summary>
        /// Draw an arrow that points from the p0 grid coordinates to p1 grid coordinates.        
        /// </summary>
        /// <param name="p1">start point</param>
        /// <param name="p2">end point</param>
        private void DrawArrow(PointF p1, PointF p2) 
        {
            GraphicsPath gp = sf [ShapeFactory.RelationshipArrow];
            double alpha = Math.Atan2(p1.Y - p2.Y, p2.X - p1.X)+ Math.PI / 2;
            RectangleF bounds = gp.GetBounds();
            float dx = (p1.X + p2.X) / 2 ;
            float dy = (p1.Y + p2.Y) / 2 ;

            // center the arrow in 0,0
            gp.Transform(new Matrix(1,0,0,1,- (bounds.Width / 2 + bounds.X), - (bounds.Height / 2 + bounds.Y)));
            gp.Transform(new Matrix((float) Math.Cos(alpha), (float) -Math.Sin(alpha), (float) Math.Sin(alpha), (float) Math.Cos(alpha),dx,dy));
            
            Pen pen= new Pen(Brushes.Black, 1);
            graphics.DrawPath(pen, gp);
        }

        /// <summary>
        /// Caclulates the size needed for the text to fit.
        /// </summary>
        /// <param name="text">text to measure</param>
        /// <returns></returns>
        public SizeF CalcShapeSize(string text) 
        {
            SizeF s = graphics.MeasureString(text,new Font(FontFamily.GenericSansSerif, 12));
            return new SizeF(s.Width + margin * 2 + 2, s.Height + margin * 2 + 2);
        }

	}
}

