/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */

package com.topcoder.rules.dependencymap.view.AccuracyTests;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.image.BufferedImage;
import java.util.HashSet;
import java.util.Set;

import com.topcoder.rules.dependencymap.view.RendererException;
import com.topcoder.rules.dependencymap.view.RendererGraphics;
import com.topcoder.util.image.svg2dgraphics.SVGGraphics2D;

/**
 * Class that logs the calls to DrawString and Draw, and also paints in a SVG and s to a file if not
 * constructed with null as the filename.
 *
 * @author cucu
 */
public class MockGraphics implements RendererGraphics {
        /**
         * This represents the underlying SVGGraphics2D that this adapter class uses. This variable is created at
         * object instantiation time, is immutable (the reference, not the contents) and is accessed by all methods.
         * This variable has been marked final to ensure it's immutability.
         */
        private final Graphics2D g2d;
    
        private BufferedImage buffImage = null;
        
        /**
         * set of method calls.
         */
        private Set log;

        /**
         * Get the log, that is a set of strings representing the method calls
         * @return a set of strings representing the method calls
         */
        public Set getLog() {
            return log;
        }

        /**
         * Create a new LogGraphics.
         */
        public MockGraphics( BufferedImage buffImg) {
            log = new HashSet();
            if (buffImg != null) {
            	buffImage = buffImg;
            	g2d = buffImage.createGraphics();
            	g2d.setBackground(Color.WHITE);
            	g2d.clearRect(0,0,buffImg.getWidth(), buffImg.getHeight());
            	g2d.setColor(Color.BLACK);
            } else {
            	g2d = new SVGGraphics2D();
            }

        }
        public MockGraphics() {
        	this(null);
        }
        /**
         * Get's the current color. This method simply returns g2d.getColor().
         *
         * @return The current color
         */
        public Color getColor() {
            return g2d.getColor();
        }

        /**
         * Set's the current color.  This method simply returns g2d.setColor(color).
         *
         * @param color The color to use
         * @throws NullPointerException if color is null
         */
        public void setColor(Color color) {
            g2d.setColor(color);
        }

        /**
         * Get's the current font. This method simply returns g2d.getFont().
         *
         * @return The current font
         */
        public Font getFont() {
            return g2d.getFont();
        }

        /**
         * Set's the current font. This method simply returns g2d.setFont(font).
         *
         * @param font The font to use
         * @throws NullPointerException if font is null
         */
        public void setFont(Font font) {
           g2d.setFont(font);
        }

        /**
         * Get's the font's metrics. This method simply returns g2d.getFontMetrics(font).
         *
         * @param font The font to get metrics for
         * @return The font metrics
         * @throws NullPointerException if font is null
         */
        public FontMetrics getFontMetrics(Font font) {
            return g2d.getFontMetrics(font);
        }

        private void logDraw(String action, Color color, Shape shape) {
            String s = action + "["+color.toString() +"] ";
            if (shape instanceof Rectangle) {
                Rectangle r = (Rectangle) shape;
                s += "Rectangle(" + r.x + ";" + r.y + ") with size (" + r.width + ";" + r.height + ")  ";
            } else {
                PathIterator it = shape.getPathIterator(null);
                while (!it.isDone()) {
                    double d[] = new double [6];
                    it.next();
                    int type = it.currentSegment(d);
                    int points = 0;

                    switch (type) {
                    case PathIterator.SEG_LINETO:
                        s+="LineTo(";
                        points = 2;
                        break;
                    case PathIterator.SEG_MOVETO:
                        s+="MoveTo(";
                        points = 2;
                        break;
                    case PathIterator.SEG_QUADTO:
                        s+="QuadTo(";
                        points = 4;
                        break;
                    case PathIterator.SEG_CUBICTO:
                        s+="CubicTo(";
                        points = 6;
                        break;
                    case PathIterator.SEG_CLOSE:
                        s+="Close(";
                        points = 0;
                        break;
                    }

                    for (int j = 0; j < points; j++) {
                        s+=(int) d[j]+( j < (points - 1) ? ";" : "");
                    }
                    s+="), ";
                }
            }
            log.add(s.substring(0, s.length() - 2));
        	
        }
        /**
         * Draws the specified shape. Simply calls g2d.draw(shape).
         *
         * @param shape The shape to draw
         * @throws NullPointerException if shape is null
         */
        public void draw(GeneralPath shape) {
            g2d.draw(shape);
            logDraw("Draw ", g2d.getColor(), shape);
        }

        public void fill(Shape shape) {
            g2d.fill(shape);
            logDraw("Fill ", g2d.getColor(), shape);
        }

        /**
         * Draws the specified string at the specified position. Simply calls g2d.drawString(str, x, y).
         *
         * @param str The text to draw
         * @param x The horizontal position to draw the text at
         * @param y The vertical position (baseline) to draw the text at
         * @throws NullPointerException if str is null
         */
        public void drawString(String str, int x, int y) {
            log.add("DrawString " + str + "[" + g2d.getFont() + "] at (" + x + ";" + y + ")");
            g2d.drawString(str, x, y);
        }

        /**
         * Completes the processing by saving the file. This method simply calls g2d.renderSVGImage(filename).
         * If any exception is thrown, catch/wrap/throw a RendererException with the exception embedded.
         *
         * @throws RendererException if an exception occurs completing processing
         */
        public void complete() throws RendererException {
        }
        
        public void save(String filename) {
    		if (g2d instanceof SVGGraphics2D) {
        		TestsHelper.saveSVG(filename + ".svg", (SVGGraphics2D) g2d);
        	} else if (g2d instanceof Graphics2D) {
        		TestsHelper.saveBMP(filename + ".bmp", buffImage);
        	}
        }
        

        /**
         * Disposes of any held resources. Simply calls g2d.dispose().
         */
        public void dispose() {
            g2d.dispose();
        }

    }
